#! /usr/bin/python

#
# This file is part of the Sierpi project. For conditions of distribution and
# use, please see the copyright notice in the file 'copyright.txt' at the root
# directory of this package and the copyright notice at
# http://www5.in.tum.de/sierpi
#

# This is the main python program to create all different kinds of traversator codes.
# The type of Code is specified on the end of this file

# !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
# !!! not all combinations for the configuration class scg.CConfig are possible!  !!!
# !!! e. g. it's not possible to combine the adaptivity part with the computation !!!
# !!! traversals                                                                  !!!
# !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

import sys
import math

# import our code generator class
import lib_sierpinski_code_generator as scg



###########################################################################################
######################################## TEST SET #########################################
###########################################################################################


###########################################################################################
############# CODE GENERATION CONFIGURATION STARTS HERE ###################################
###########################################################################################

#####################################################################################################################
#####################################################################################################################
# LEAF ONLY
#####################################################################################################################
#####################################################################################################################

# debug output is written to STDERR! STDOUT is redirected to the C++ header file
sys.stderr.write("VERTICES DEPTH\n\n")
# VERTICES DEPTH
if True:
	sys.stdout = open("../traversators/vertices/auto/CTraversator_Vertices_Depth_autogenerated.hpp", 'w')

	# create a fresh instance
	scgrun = scg.scg()

	# create a config class with default parameters
	config = scg.CConfig()
	# create vertices
	config.vertices = True
	# create the code for the forward traversals
	config.createForwardCode = True
	# create the code for the backward traversals
	config.createBackwardCode = True
	# create code including the depth as a parameter for kernel hooks
	config.depthParameter = True
	# the kernels hooks are only called at leaf nodes
	config.kernelCallsForLeavesOnly = True
	# assume that K, V and H types are equal and can be replaced by 'U' (unified)
	config.assumeKVHToBeEqual = True

	# create the c++ code!
	scgrun.createCode(config)

	# restore stdout
	sys.stdout = sys.__stdout__

sys.stderr.write("VERTICES\n\n")
# VERTICES
if True:
	sys.stdout = open("../traversators/vertices/auto/CTraversator_Vertices_autogenerated.hpp", 'w')

	# create a fresh instance
	scgrun = scg.scg()
	
	config = scg.CConfig()
	config.vertices = True
	config.createForwardCode = True
	config.createBackwardCode = True
	config.kernelCallsForLeavesOnly = True
	config.createRecursionTable = True
	config.limitRecursionTableMethodsByDepth = 2
	config.createSplitJoinFirstChildInformation = True
#	# assume that K, V and H types are equal and can be replaced by 'U' (unified)
	config.assumeKVHToBeEqual = True


	scgrun.createCode(config)

	scgrun.outputRecursionTable(config)

	sys.stdout = sys.__stdout__


sys.stderr.write("VERTICES & ELEMENT DATA\n\n")
# VERTICES & ELEMENT DATA
if True:
	sys.stdout = open("../traversators/vertices/auto/CTraversator_Vertices_ElementData_autogenerated.hpp", 'w')

	# create a fresh instance
	scgrun = scg.scg()
	
	config = scg.CConfig()
	config.vertices = True
	config.elementData = True
	config.createForwardCode = True
	config.createBackwardCode = True
	config.kernelCallsForLeavesOnly = True

	config.createRecursionTable = True
	config.createSplitJoinFirstChildInformation = True
	config.limitRecursionTableMethodsByDepth = 2
	# assume that K, V and H types are equal and can be replaced by 'U' (unified)
	config.assumeKVHToBeEqual = True

	scgrun.createCode(config)

	scgrun.outputRecursionTable(config)

	sys.stdout = sys.__stdout__


sys.stderr.write("VERTICES & ELEMENT DATA & DEPTH\n\n")
# VERTICES & ELEMENT DATA
if True:
	sys.stdout = open("../traversators/vertices/auto/CTraversator_Vertices_ElementData_Depth_autogenerated.hpp", 'w')

	# create a fresh instance
	scgrun = scg.scg()
	
	config = scg.CConfig()
	config.vertices = True
	config.elementData = True
	config.depthParameter = True
	config.createForwardCode = True
	config.createBackwardCode = True
	config.kernelCallsForLeavesOnly = True

	config.createRecursionTable = True
	config.createSplitJoinFirstChildInformation = True
	config.limitRecursionTableMethodsByDepth = 2
	# assume that K, V and H types are equal and can be replaced by 'U' (unified)
	config.assumeKVHToBeEqual = True

	scgrun.createCode(config)

	scgrun.outputRecursionTable(config)

	sys.stdout = sys.__stdout__



sys.stderr.write("VERTICES & NORMALS\n\n")
# VERTICES & NORMALS
if False:
	sys.stdout = open("../traversators/vertices/auto/CTraversator_Vertices_Normals_autogenerated.hpp", 'w')

	# create a fresh instance
	scgrun = scg.scg()
	
	config = scg.CConfig()
	config.vertices = True
	config.normals = True
	config.createForwardCode = True
	config.createBackwardCode = True
	config.depthParameter = False
	config.kernelCallsForLeavesOnly = True

	config.createRecursionTable = True
	config.createSplitJoinFirstChildInformation = True
	config.limitRecursionTableMethodsByDepth = 2
	# assume that K, V and H types are equal and can be replaced by 'U' (unified)
	config.assumeKVHToBeEqual = True

	scgrun.createCode(config)

	scgrun.outputRecursionTable(config)

	sys.stdout = sys.__stdout__


sys.stderr.write("VERTICES & NORMALS & ELEMENT DATA\n\n")
# VERTICES & NORMALS & ELEMENT DATA
if True:
	sys.stdout = open("../traversators/vertices/auto/CTraversator_Vertices_Normals_ElementData_autogenerated.hpp", 'w')

	# create a fresh instance
	scgrun = scg.scg()
	
	config = scg.CConfig()
	config.vertices = True
	config.normals = True
	config.elementData = True
	config.createForwardCode = True
	config.createBackwardCode = True
	config.depthParameter = False
	config.kernelCallsForLeavesOnly = True

	config.createRecursionTable = True
	config.createSplitJoinFirstChildInformation = True
	config.limitRecursionTableMethodsByDepth = 2
	# assume that K, V and H types are equal and can be replaced by 'U' (unified)
	config.assumeKVHToBeEqual = True

	scgrun.createCode(config)

	scgrun.outputRecursionTable(config)

	sys.stdout = sys.__stdout__


#####################################################################################################################
#####################################################################################################################
# ADAPTIVE
#####################################################################################################################
#####################################################################################################################



sys.stderr.write("ADAPTIVE with ELEMENTS and NORMALS and DEPTH and DEPTH LIMITERS min/max PARALLEL PART (first)\n\n")
if True:
	# ADAPTIVE with ELEMENTS and NORMALS and DEPTH and DEPTH LIMITERS min/max (first traversal only)
	sys.stdout = open("../traversators/adaptive/auto/CAdaptiveConformingGrid_Normals_Depth_DepthLimiters_Element_FirstTraversal_autogenerated.hpp", 'w')

	# create a fresh instance
	scgrun = scg.scg()
	
	config = scg.CConfig()
	config.adaptive = True
	config.createForwardCode = True
	config.createBackwardCode = False
	config.normals = True
	config.elementData = True
	config.depthParameter = True
	config.vertices = False
	config.depthLimiterMax = True
	config.depthLimiterMin = True
	config.kernelCallsForLeavesOnly = True
	config.createRecursionTable = True
	config.createSplitJoinFirstChildInformation = True
	config.limitRecursionTableMethodsByDepth = 2
	# FIRST TRAVERSAL ONLY!
	config.adaptive_create_first_traversal = True
	config.adaptive_create_middle_traversal = False
	config.adaptive_create_last_traversal = False
	config.createSplitJoinFirstChildInformation = True
	# assume that K, V and H types are equal and can be replaced by 'U' (unified)
	config.assumeKVHToBeEqual = True

	scgrun.createCode(config)
	scgrun.outputRecursionTable(config)


	# ADAPTIVE with ELEMENTS and NORMALS and DEPTH and DEPTH LIMITERS min/max (middle traversal only)
	sys.stdout = open("../traversators/adaptive/auto/CAdaptiveConformingGrid_Normals_Depth_DepthLimiters_Element_MiddleTraversal_autogenerated.hpp", 'w')

	# create a fresh instance
	scgrun = scg.scg()
	
	config = scg.CConfig()
	config.adaptive = True
	config.createForwardCode = True
	config.createBackwardCode = True
	config.normals = False
	config.elementData = False
	config.depthParameter = False
	config.vertices = False
	config.kernelCallsForLeavesOnly = True
	config.createRecursionTable = True
	config.createSplitJoinFirstChildInformation = True
	config.limitRecursionTableMethodsByDepth = 2
	# MIDDLE TRAVERSAL ONLY!
	config.adaptive_create_first_traversal = False
	config.adaptive_create_middle_traversal = True
	config.adaptive_create_last_traversal = False
	# deactivate asserts which can be used for serial processing only
	# there are parallel exchange races (adjacent triangles get both refined) which avoid using the asserts
	config.adaptive_middle_create_serial_processing_asserts = False
	config.createSplitJoinFirstChildInformation = True
	# assume that K, V and H types are equal and can be replaced by 'U' (unified)
	config.assumeKVHToBeEqual = True

	scgrun.createCode(config)
	scgrun.outputRecursionTable(config)
	

	# ADAPTIVE with ELEMENTS and NORMALS and DEPTH and DEPTH LIMITERS min/max (last traversal only)
	sys.stdout = open("../traversators/adaptive/auto/CAdaptiveConformingGrid_Normals_Depth_DepthLimiters_Element_LastTraversal_autogenerated.hpp", 'w')
	# create a fresh instance
	scgrun = scg.scg()
	
	config = scg.CConfig()
	config.adaptive = True
	config.createForwardCode = False
	config.createBackwardCode = True
 	config.normals = True
	config.elementData = True
	config.depthParameter = True
	config.updateMaxDepth = True
	config.vertices = False
	config.kernelCallsForLeavesOnly = True
	config.createRecursionTable = True
	config.createSplitJoinFirstChildInformation = True
	config.limitRecursionTableMethodsByDepth = 2

	# LAST TRAVERSAL ONLY!
	config.adaptive_create_first_traversal = False
	config.adaptive_create_middle_traversal = False
	config.adaptive_create_last_traversal = True

	# return value of methods has to be of type 'char' to spread coarsening information
	config.adaptive_char_return_variable = True
	# output refinement information to the communication stacks
	config.adaptive_last_traversal_edge_comm_adaptive_information = True
	# assume that K, V and H types are equal and can be replaced by 'U' (unified)
	config.assumeKVHToBeEqual = True

	scgrun.createCode(config)
	scgrun.outputRecursionTable(config)

	sys.stdout = sys.__stdout__



sys.stderr.write("ADAPTIVE with VERTICES DEPTH LIMITERS min/max ELEMENT DATA PARALLEL PART (first)\n\n")
if True:
	# ADAPTIVE with VERTICES DEPTH LIMITERS min/max ELEMENT DATA PARALLEL PART (first traversal only)
	sys.stdout = open("../traversators/adaptive/auto/CAdaptiveConformingGrid_Vertices_DepthLimiters_Element_FirstTraversal_autogenerated.hpp", 'w')

	# create a fresh instance
	scgrun = scg.scg()
	
	config = scg.CConfig()
	config.adaptive = True
	config.createForwardCode = True
	config.createBackwardCode = False
	config.normals = False
	config.elementData = True
	config.depthParameter = False
	config.vertices = True
	config.depthLimiterMax = True
	config.depthLimiterMin = True
	config.kernelCallsForLeavesOnly = True
	config.createRecursionTable = True
	config.createSplitJoinFirstChildInformation = True
	config.limitRecursionTableMethodsByDepth = 2
	config.callShouldRefineForAllLeaves = True
	
	# FIRST TRAVERSAL ONLY!
	config.adaptive_create_first_traversal = True
	config.adaptive_create_middle_traversal = False
	config.adaptive_create_last_traversal = False
	config.createSplitJoinFirstChildInformation = True
	
	# assume that K, V and H types are equal and can be replaced by 'U' (unified)
	config.assumeKVHToBeEqual = True

	scgrun.createCode(config)
	scgrun.outputRecursionTable(config)


	# ADAPTIVE with ELEMENTS and NORMALS and DEPTH and DEPTH LIMITERS min/max (middle traversal only)
	sys.stdout = open("../traversators/adaptive/auto/CAdaptiveConformingGrid_Vertices_DepthLimiters_Element_MiddleTraversal_autogenerated.hpp", 'w')

	# create a fresh instance
	scgrun = scg.scg()
	
	config = scg.CConfig()
	config.adaptive = True
	config.createForwardCode = True
	config.createBackwardCode = True
	config.normals = False
	config.elementData = False
	config.depthParameter = False
	config.vertices = False
	config.kernelCallsForLeavesOnly = True
	config.createRecursionTable = True
	config.createSplitJoinFirstChildInformation = True
	config.limitRecursionTableMethodsByDepth = 2
	# MIDDLE TRAVERSAL ONLY!
	config.adaptive_create_first_traversal = False
	config.adaptive_create_middle_traversal = True
	config.adaptive_create_last_traversal = False
	# deactivate asserts which can be used for serial processing only
	# there are parallel exchange races (adjacent triangles get both refined) which avoid using the asserts
	config.adaptive_middle_create_serial_processing_asserts = False
	config.createSplitJoinFirstChildInformation = True
	# assume that K, V and H types are equal and can be replaced by 'U' (unified)
	config.assumeKVHToBeEqual = True

	scgrun.createCode(config)	
	scgrun.outputRecursionTable(config)


	# ADAPTIVE with ELEMENTS and NORMALS and DEPTH and DEPTH LIMITERS min/max (last traversal only)
	sys.stdout = open("../traversators/adaptive/auto/CAdaptiveConformingGrid_Vertices_DepthLimiters_Element_LastTraversal_autogenerated.hpp", 'w')
	# create a fresh instance
	scgrun = scg.scg()
	
	config = scg.CConfig()
	config.adaptive = True
	config.createForwardCode = False
	config.createBackwardCode = True
 	config.normals = False
	config.elementData = True
	config.depthParameter = False
	config.updateMaxDepth = True
	config.vertices = True
	config.kernelCallsForLeavesOnly = True
	config.createRecursionTable = True
	config.createSplitJoinFirstChildInformation = True
	config.limitRecursionTableMethodsByDepth = 2

	# LAST TRAVERSAL ONLY!
	config.adaptive_create_first_traversal = False
	config.adaptive_create_middle_traversal = False
	config.adaptive_create_last_traversal = True
	
	# return value of methods has to be of type 'char' to spread coarsening information
	config.adaptive_char_return_variable = True
	# output refinement information to the communication stacks
	config.adaptive_last_traversal_edge_comm_adaptive_information = True

	# assume that K, V and H types are equal and can be replaced by 'U' (unified)
	config.assumeKVHToBeEqual = True
	
	scgrun.createCode(config)
	scgrun.outputRecursionTable(config)

	sys.stdout = sys.__stdout__


#####################################################################################################################
#####################################################################################################################
# EDGE COMM
#####################################################################################################################
#####################################################################################################################

sys.stderr.write("EDGE COMM\n\n")
# COMPUTATION: EDGE COMM
if True:
	sys.stdout = open("../traversators/edgeComm/auto/CTraversator_EdgeComm_autogenerated.hpp", 'w')

	# create a fresh instance
	scgrun = scg.scg()
	
	config = scg.CConfig()
	config.edgeComm = True
	config.elementData = True
	config.createForwardCode = True
	config.kernelCallsForLeavesOnly = True
	config.createReversedStructureStack = True
	config.createSplitJoinFirstChildInformation = True
	# assume that K, V and H types are equal and can be replaced by 'U' (unified)
	config.assumeKVHToBeEqual = True

	scgrun.createCode(config)

	sys.stdout = open("../traversators/edgeComm/auto/CTraversator_EdgeComm_autogenerated.hpp", 'a')

	# create a fresh instance
	scgrun = scg.scg()
	
	config = scg.CConfig()
	config.edgeComm = True
	config.elementData = True
	config.createBackwardCode = True
	config.kernelCallsForLeavesOnly = True
	config.element_data_input_stack = 'element_data_fifo'
	config.createSplitJoinFirstChildInformation = True
	# assume that K, V and H types are equal and can be replaced by 'U' (unified)
	config.assumeKVHToBeEqual = True

	scgrun.createCode(config)

	sys.stdout = sys.__stdout__


sys.stderr.write("EDGE COMM + NORMALS\n\n")
# COMPUTATION: EDGE COMM + NORMALS	
if True:
	sys.stdout = open("../traversators/edgeComm/auto/CTraversator_EdgeComm_Normals_autogenerated.hpp", 'w')

	# create a fresh instance
	scgrun = scg.scg()
	
	config = scg.CConfig()
	config.edgeComm = True
	config.elementData = True
	config.createForwardCode = True
	config.normals = True
	config.kernelCallsForLeavesOnly = True
	config.createReversedStructureStack = True
	config.createSplitJoinFirstChildInformation = True
	# assume that K, V and H types are equal and can be replaced by 'U' (unified)
	config.assumeKVHToBeEqual = True

	scgrun.createCode(config)

	sys.stdout = open("../traversators/edgeComm/auto/CTraversator_EdgeComm_Normals_autogenerated.hpp", 'a')

	# create a fresh instance
	scgrun = scg.scg()
	
	config = scg.CConfig()
	config.edgeComm = True
	config.elementData = True
	config.createBackwardCode = True
	config.normals = True
	config.kernelCallsForLeavesOnly = True
	config.element_data_input_stack = 'element_data_fifo'
	config.createSplitJoinFirstChildInformation = True
	# assume that K, V and H types are equal and can be replaced by 'U' (unified)
	config.assumeKVHToBeEqual = True

	scgrun.createCode(config)

	sys.stdout = sys.__stdout__


sys.stderr.write("EDGE COMM + NORMALS + DEPTH + recursionMethod Table\n\n")
# COMPUTATION: EDGE COMM + NORMALS + DEPTH
if True:
	sys.stdout = open("../traversators/edgeComm/auto/CTraversator_EdgeComm_Normals_Depth_autogenerated.hpp", 'w')

	# create a fresh instance
	scgrun = scg.scg()
	
	config = scg.CConfig()
	config.edgeComm = True
	config.elementData = True
	config.createForwardCode = True
	config.normals = True
	config.depthParameter = True
	config.kernelCallsForLeavesOnly = True
	config.createReversedStructureStack = True
	config.createRecursionTable = True
	config.createSplitJoinFirstChildInformation = True
	config.limitRecursionTableMethodsByDepth = 2
	# assume that K, V and H types are equal and can be replaced by 'U' (unified)
	config.assumeKVHToBeEqual = True

	scgrun.createCode(config)
	
	# backup recursion table
	recTable = scgrun.recursionTable
	
	sys.stdout = open("../traversators/edgeComm/auto/CTraversator_EdgeComm_Normals_Depth_autogenerated.hpp", 'a')

	# create a fresh instance
	scgrun = scg.scg()
	
	# restore recursion table
	scgrun.recursionTable = recTable
	
	config = scg.CConfig()
	config.edgeComm = True
	config.elementData = True
	config.createBackwardCode = True
	config.normals = True
	config.depthParameter = True
	config.kernelCallsForLeavesOnly = True
	config.element_data_input_stack = 'element_data_fifo'
	config.methodRecursionTable = True
	config.createRecursionTable = True
	config.limitRecursionTableMethodsByDepth = 2
	config.createSplitJoinFirstChildInformation = True
	# assume that K, V and H types are equal and can be replaced by 'U' (unified)
	config.assumeKVHToBeEqual = True

	scgrun.createCode(config)

	scgrun.outputRecursionTable(config)

	sys.stdout = sys.__stdout__


sys.stderr.write("COMPUTATION: EDGE COMM + NORMALS + VERTICES\n\n")
# COMPUTATION: EDGE COMM + NORMALS + VERTICES
if True:
	sys.stdout = open("../traversators/edgeComm/auto/CTraversator_EdgeComm_Vertices_Normals_autogenerated.hpp", 'w')

	# create a fresh instance
	scgrun = scg.scg()
	
	config = scg.CConfig()
	config.edgeComm = True
	config.elementData = True
	config.createForwardCode = True
	config.normals = True
	config.vertices = True
	config.kernelCallsForLeavesOnly = True
	config.createReversedStructureStack = True
	config.createRecursionTable = True
	config.limitRecursionTableMethodsByDepth = 2
	config.createSplitJoinFirstChildInformation = True
	# assume that K, V and H types are equal and can be replaced by 'U' (unified)
	config.assumeKVHToBeEqual = True

	scgrun.createCode(config)

	# backup recursion table
	recTable = scgrun.recursionTable
	
	sys.stdout = open("../traversators/edgeComm/auto/CTraversator_EdgeComm_Vertices_Normals_autogenerated.hpp", 'a')
	
	# create a fresh instance
	scgrun = scg.scg()
	
	# restore recursion table
	scgrun.recursionTable = recTable
	
	config = scg.CConfig()
	config.edgeComm = True
	config.elementData = True
	config.createBackwardCode = True
	config.normals = True
	config.vertices = True
	config.kernelCallsForLeavesOnly = True
	config.element_data_input_stack = 'element_data_fifo'
	config.methodRecursionTable = True
	config.createRecursionTable = True
	config.limitRecursionTableMethodsByDepth = 2
	config.createSplitJoinFirstChildInformation = True
	# assume that K, V and H types are equal and can be replaced by 'U' (unified)
	config.assumeKVHToBeEqual = True

	scgrun.createCode(config)
	
	scgrun.outputRecursionTable(config)

	sys.stdout = sys.__stdout__

