/*
 * Copyright (C) 2011 Technische Universitaet Muenchen
 * This file is part of the Sierpi project. For conditions of distribution and
 * use, please see the copyright notice in the file 'copyright.txt' at the root
 * directory of this package and the copyright notice at http://www5.in.tum.de/sierpi
 *
 *
 *  Created on: Jan 10, 2011
 *      Author: Martin Schreiber (martin.schreiber@in.tum.de)
 */

#ifndef CTRAVERSATOR_VERTICES_ELEMENTDATA_DEPTH_HPP_
#define CTRAVERSATOR_VERTICES_ELEMENTDATA_DEPTH_HPP_

#include "libsierpi/stacks/CSimulationStacks.hpp"
#include "libsierpi/triangle/CTriangle_Factory.hpp"

namespace sierpi
{
namespace travs
{


template <class CKernelClass, typename CSimulationStacks>
class CTraversator_Vertices_ElementData_Depth
{
	typedef typename CSimulationStacks::TElementData TElementData;

public:
	/**
	 * implementation of kernel class
	 */
	CKernelClass kernelClass;

	/**
	 * a few typedefs which could be utilized by the traversator to get more informations
	 */
	typedef typename CKernelClass::TVertexScalar TVertexScalar;
	typedef CTraversator_Vertices_ElementData_Depth<CKernelClass, CSimulationStacks> TThisClass;


	/**
	 * stack and lifo handlers used in this traversator
	 */
	CStackReaderTopDown<char> structure_lifo_in;
	CStackReaderTopDown<TElementData> element_data_lifo;

	/**
	 * include automagically generated code
	 */
#define SIERPI_PARALLEL_CODE	1
	#include "auto/CTraversator_Vertices_ElementData_Depth_autogenerated.hpp"
#undef SIERPI_PARALLEL_CODE
#include "../CTraversatorClassInc_SetupSinglePass.hpp"


	/**
	 * do the traversal!
	 */
	void action(
			CSimulationStacks *io_simulationStacks
	)
	{
		assert(io_simulationStacks->structure_stacks.direction == io_simulationStacks->element_data_stacks.direction);
		assert(io_simulationStacks->structure_stacks.direction == CFBStacks<char>::FORWARD);

		structure_lifo_in.setup(io_simulationStacks->structure_stacks.forward);
		element_data_lifo.setup(io_simulationStacks->element_data_stacks.forward);

		kernelClass.traversal_pre_hook();

		(this->*sfcRecursiveMethod_Forward)(
				triangleFactory.vertices[0][0], triangleFactory.vertices[0][1],
				triangleFactory.vertices[1][0], triangleFactory.vertices[1][1],
				triangleFactory.vertices[2][0], triangleFactory.vertices[2][1],
				triangleFactory.recursionDepthFirstRecMethod
			);

		kernelClass.traversal_post_hook();

		assert(element_data_lifo.isEmpty());
		assert(structure_lifo_in.isEmpty());
	}
};

}
}

#endif /* CSTRUCTURE_VERTEXDATA_H_ */
