/*
 * Copyright (C) 2011 Technische Universitaet Muenchen
 * This file is part of the Sierpi project. For conditions of distribution and
 * use, please see the copyright notice in the file 'copyright.txt' at the root
 * directory of this package and the copyright notice at http://www5.in.tum.de/sierpi
 *
 * CEdgeComm_Tsunami_0thOrder_BoundaryConditions.hpp
 *
 *  Created on: Nov 6, 2011
 *      Author: Martin Schreiber (martin.schreiber@in.tum.de)
 */

#ifndef CEDGECOMM_TSUNAMI_0THORDER_BOUNDARYCONDITIONS_HPP_
#define CEDGECOMM_TSUNAMI_0THORDER_BOUNDARYCONDITIONS_HPP_



inline void CEdgeComm_Tsunami_0thOrder::elementAction_BBB(
		TVertexScalar hyp_normal_x, TVertexScalar hyp_normal_y,
		TVertexScalar right_normal_x, TVertexScalar right_normal_y,
		TVertexScalar left_normal_x, TVertexScalar left_normal_y,
		int depth,
		CTsunamiElementData *elementData
)
{
	CTsunamiEdgeData hyp_edge;
	CTsunamiEdgeData left_edge;
	CTsunamiEdgeData right_edge;

	switch(eBoundaryCondition)
	{
	case BOUNDARY_CONDITION_DIRICHLET:
		left_edge = config.boundary_dirichlet;
		break;

	case BOUNDARY_CONDITION_VELOCITY_DAMPING:
		left_edge = elementData->dofs;
		break;

	case BOUNDARY_CONDITION_BOUNCE_BACK:
		left_edge.h = elementData->dofs.h;
		left_edge.qx = -elementData->dofs.qx;
		left_edge.qy = -elementData->dofs.qy;

		right_edge.h = elementData->dofs.h;
		right_edge.qx = -elementData->dofs.qx;
		right_edge.qy = -elementData->dofs.qy;

		hyp_edge.h = elementData->dofs.h;
		hyp_edge.qx = -elementData->dofs.qx;
		hyp_edge.qy = -elementData->dofs.qy;
		break;
	}

#if COMPILE_WITH_DEBUG_TSUNAMI_ELEMENTDATA_VALIDATION
	hyp_edge.validation.setupNormal(-hyp_normal_x, -hyp_normal_y);
	right_edge.validation.setupNormal(-right_normal_x, -right_normal_y);
	left_edge.validation.setupNormal(-left_normal_x, -left_normal_y);

	elementData->validation.setupHypEdgeCommData(&hyp_edge.validation);
	elementData->validation.setupRightEdgeCommData(&right_edge.validation);
	elementData->validation.setupLeftEdgeCommData(&left_edge.validation);
#endif

	elementAction_EEE(	hyp_normal_x, hyp_normal_y,
						right_normal_x, right_normal_y,
						left_normal_x, left_normal_y,
						depth,
						elementData,
						&hyp_edge, &right_edge, &left_edge);
}


inline void CEdgeComm_Tsunami_0thOrder::elementAction_BEE(
		TVertexScalar hyp_normal_x, TVertexScalar hyp_normal_y,
		TVertexScalar right_normal_x, TVertexScalar right_normal_y,
		TVertexScalar left_normal_x, TVertexScalar left_normal_y,
		int depth,
		CTsunamiElementData *elementData,
		CTsunamiEdgeData *right_edge,
		CTsunamiEdgeData *left_edge
)
{
	CTsunamiEdgeData hyp_edge;

	switch(eBoundaryCondition)
	{
	case BOUNDARY_CONDITION_DIRICHLET:
		hyp_edge = config.boundary_dirichlet;
		break;

	case BOUNDARY_CONDITION_VELOCITY_DAMPING:
		hyp_edge = elementData->dofs;
		break;

	case BOUNDARY_CONDITION_BOUNCE_BACK:
		hyp_edge.h = elementData->dofs.h;
		hyp_edge.qx = -elementData->dofs.qx;
		hyp_edge.qy = -elementData->dofs.qy;
		break;
	}

#if COMPILE_WITH_DEBUG_TSUNAMI_ELEMENTDATA_VALIDATION
	hyp_edge.validation.setupNormal(-hyp_normal_x, -hyp_normal_y);

	elementData->validation.setupHypEdgeCommData(&hyp_edge.validation);
#endif

	elementAction_EEE(
			hyp_normal_x, hyp_normal_y,
			right_normal_x, right_normal_y,
			left_normal_x, left_normal_y,
			depth,
			elementData,
			&hyp_edge, right_edge, left_edge);
}


inline void CEdgeComm_Tsunami_0thOrder::elementAction_EBE(
		TVertexScalar hyp_normal_x, TVertexScalar hyp_normal_y,
		TVertexScalar right_normal_x, TVertexScalar right_normal_y,
		TVertexScalar left_normal_x, TVertexScalar left_normal_y,
		int depth,
		CTsunamiElementData *elementData,
		CTsunamiEdgeData *hyp_edge,
		CTsunamiEdgeData *left_edge
)
{
	CTsunamiEdgeData right_edge;

	switch(eBoundaryCondition)
	{
	case BOUNDARY_CONDITION_DIRICHLET:
		right_edge = config.boundary_dirichlet;
		break;

	case BOUNDARY_CONDITION_VELOCITY_DAMPING:
		right_edge = elementData->dofs;
		break;

	case BOUNDARY_CONDITION_BOUNCE_BACK:
		right_edge.h = elementData->dofs.h;
		right_edge.qx = -elementData->dofs.qx;
		right_edge.qy = -elementData->dofs.qy;
		break;
	}

#if COMPILE_WITH_DEBUG_TSUNAMI_ELEMENTDATA_VALIDATION
	right_edge.validation.setupNormal(-right_normal_x, -right_normal_y);

	elementData->validation.setupRightEdgeCommData(&right_edge.validation);
#endif

	elementAction_EEE(
			hyp_normal_x, hyp_normal_y,
			right_normal_x, right_normal_y,
			left_normal_x, left_normal_y,
			depth,
			elementData,
			hyp_edge, &right_edge, left_edge);
}


inline void CEdgeComm_Tsunami_0thOrder::elementAction_EEB(
		TVertexScalar hyp_normal_x, TVertexScalar hyp_normal_y,
		TVertexScalar right_normal_x, TVertexScalar right_normal_y,
		TVertexScalar left_normal_x, TVertexScalar left_normal_y,
		int depth,
		CTsunamiElementData *elementData,
		CTsunamiEdgeData *hyp_edge,
		CTsunamiEdgeData *right_edge
)
{
	CTsunamiEdgeData left_edge;

	switch(eBoundaryCondition)
	{
	case BOUNDARY_CONDITION_DIRICHLET:
		left_edge = config.boundary_dirichlet;
		break;

	case BOUNDARY_CONDITION_VELOCITY_DAMPING:
		left_edge = elementData->dofs;
		break;

	case BOUNDARY_CONDITION_BOUNCE_BACK:
		left_edge.h = elementData->dofs.h;
		left_edge.qx = -elementData->dofs.qx;
		left_edge.qy = -elementData->dofs.qy;
		break;
	}

#if COMPILE_WITH_DEBUG_TSUNAMI_ELEMENTDATA_VALIDATION
	left_edge.validation.setupNormal(-left_normal_x, -left_normal_y);

	elementData->validation.setupLeftEdgeCommData(&left_edge.validation);
#endif

	elementAction_EEE(
			hyp_normal_x, hyp_normal_y,
			right_normal_x, right_normal_y,
			left_normal_x, left_normal_y,
			depth,
			elementData,
			hyp_edge, right_edge, &left_edge);
}


inline void CEdgeComm_Tsunami_0thOrder::elementAction_EBB(
		TVertexScalar hyp_normal_x, TVertexScalar hyp_normal_y,
		TVertexScalar right_normal_x, TVertexScalar right_normal_y,
		TVertexScalar left_normal_x, TVertexScalar left_normal_y,
		int depth,
		CTsunamiElementData *elementData,
		CTsunamiEdgeData *hyp_edge
)
{
	CTsunamiEdgeData left_edge;
	CTsunamiEdgeData right_edge;

	switch(eBoundaryCondition)
	{
	case BOUNDARY_CONDITION_DIRICHLET:
		left_edge = config.boundary_dirichlet;
		right_edge = config.boundary_dirichlet;
		break;

	case BOUNDARY_CONDITION_VELOCITY_DAMPING:
		left_edge = elementData->dofs;
		right_edge = elementData->dofs;
		break;

	case BOUNDARY_CONDITION_BOUNCE_BACK:
		left_edge.h = elementData->dofs.h;
		left_edge.qx = -elementData->dofs.qx;
		left_edge.qy = -elementData->dofs.qy;

		right_edge.h = elementData->dofs.h;
		right_edge.qx = -elementData->dofs.qx;
		right_edge.qy = -elementData->dofs.qy;

		break;
	}

#if COMPILE_WITH_DEBUG_TSUNAMI_ELEMENTDATA_VALIDATION
	right_edge.validation.setupNormal(-right_normal_x, -right_normal_y);
	left_edge.validation.setupNormal(-left_normal_x, -left_normal_y);

	elementData->validation.setupRightEdgeCommData(&right_edge.validation);
	elementData->validation.setupLeftEdgeCommData(&left_edge.validation);
#endif


	elementAction_EEE(	hyp_normal_x, hyp_normal_y,
						right_normal_x, right_normal_y,
						left_normal_x, left_normal_y,
						depth,
						elementData,
						hyp_edge, &right_edge, &left_edge);
}



inline void CEdgeComm_Tsunami_0thOrder::elementAction_BEB(
		TVertexScalar hyp_normal_x, TVertexScalar hyp_normal_y,
		TVertexScalar right_normal_x, TVertexScalar right_normal_y,
		TVertexScalar left_normal_x, TVertexScalar left_normal_y,
		int depth,
		CTsunamiElementData *elementData,
		CTsunamiEdgeData *right_edge
)
{
	CTsunamiEdgeData hyp_edge;
	CTsunamiEdgeData left_edge;

	hyp_edge = config.boundary_dirichlet;
	left_edge = config.boundary_dirichlet;

	switch(eBoundaryCondition)
	{
	case BOUNDARY_CONDITION_DIRICHLET:
		left_edge = config.boundary_dirichlet;
		hyp_edge = config.boundary_dirichlet;
		break;

	case BOUNDARY_CONDITION_VELOCITY_DAMPING:
		left_edge = elementData->dofs;
		hyp_edge = elementData->dofs;
		break;

	case BOUNDARY_CONDITION_BOUNCE_BACK:
		left_edge.h = elementData->dofs.h;
		left_edge.qx = -elementData->dofs.qx;
		left_edge.qy = -elementData->dofs.qy;

		hyp_edge.h = elementData->dofs.h;
		hyp_edge.qx = -elementData->dofs.qx;
		hyp_edge.qy = -elementData->dofs.qy;
		break;
	}

#if COMPILE_WITH_DEBUG_TSUNAMI_ELEMENTDATA_VALIDATION
	hyp_edge.validation.setupNormal(-hyp_normal_x, -hyp_normal_y);
	left_edge.validation.setupNormal(-left_normal_x, -left_normal_y);

	elementData->validation.setupHypEdgeCommData(&hyp_edge.validation);
	elementData->validation.setupLeftEdgeCommData(&left_edge.validation);
#endif

	elementAction_EEE(	hyp_normal_x, hyp_normal_y,
						right_normal_x, right_normal_y,
						left_normal_x, left_normal_y,
						depth,
						elementData,
						&hyp_edge, right_edge, &left_edge);
}



inline void CEdgeComm_Tsunami_0thOrder::elementAction_BBE(
		TVertexScalar hyp_normal_x, TVertexScalar hyp_normal_y,
		TVertexScalar right_normal_x, TVertexScalar right_normal_y,
		TVertexScalar left_normal_x, TVertexScalar left_normal_y,
		int depth,
		CTsunamiElementData *elementData,
		CTsunamiEdgeData *left_edge
)
{
	CTsunamiEdgeData hyp_edge;
	CTsunamiEdgeData right_edge;

	switch(eBoundaryCondition)
	{
	case BOUNDARY_CONDITION_DIRICHLET:
		right_edge = config.boundary_dirichlet;
		hyp_edge = config.boundary_dirichlet;
		break;

	case BOUNDARY_CONDITION_VELOCITY_DAMPING:
		right_edge = elementData->dofs;
		hyp_edge = elementData->dofs;
		break;

	case BOUNDARY_CONDITION_BOUNCE_BACK:
		right_edge.h = elementData->dofs.h;
		right_edge.qx = -elementData->dofs.qx;
		right_edge.qy = -elementData->dofs.qy;

		hyp_edge.h = elementData->dofs.h;
		hyp_edge.qx = -elementData->dofs.qx;
		hyp_edge.qy = -elementData->dofs.qy;
		break;
	}

#if COMPILE_WITH_DEBUG_TSUNAMI_ELEMENTDATA_VALIDATION
	hyp_edge.validation.setupNormal(-hyp_normal_x, -hyp_normal_y);
	right_edge.validation.setupNormal(-right_normal_x, -right_normal_y);

	elementData->validation.setupHypEdgeCommData(&hyp_edge.validation);
	elementData->validation.setupRightEdgeCommData(&right_edge.validation);
#endif

	elementAction_EEE(	hyp_normal_x, hyp_normal_y,
						right_normal_x, right_normal_y,
						left_normal_x, left_normal_y,
						depth,
						elementData,
						&hyp_edge, &right_edge, left_edge);
}



#endif /* CEDGECOMM_TSUNAMI_1STORDER_BOUNDARYCONDITIONS_HPP_ */
